Package: epivizr
Maintainer: Hector Corrada Bravo <hcorrada@gmail.com>
Author: Hector Corrada Bravo, Florin Chelaru, Llewellyn Smith, Naomi
    Goldstein, Jayaram Kancherla, Morgan Walter, Brian Gottfried
Version: 2.42.0
License: Artistic-2.0
Title: R Interface to epiviz web app
Description: This package provides connections to the
    epiviz web app (http://epiviz.cbcb.umd.edu) for interactive
    visualization of genomic data. Objects in R/bioc interactive
    sessions can be displayed in genome browser tracks or plots to
    be explored by navigation through genomic regions. Fundamental
    Bioconductor data structures are supported (e.g., GenomicRanges
    and RangedSummarizedExperiment objects), while providing an easy
    mechanism to support other data structures (through package epivizrData).
    Visualizations (using d3.js) can be easily added to the web app as well.
VignetteBuilder: knitr
Depends: R (>= 3.5.0), methods
Imports: epivizrServer (>= 1.1.1), epivizrData (>= 1.3.4),
        GenomicRanges, S4Vectors, IRanges, bumphunter, GenomeInfoDb
Suggests: testthat, roxygen2, knitr, Biobase, SummarizedExperiment,
        antiProfilesData, hgu133plus2.db, Mus.musculus, BiocStyle,
        minfi, rmarkdown
Collate: 'EpivizChartMgr-class.R' 'EpivizApp-class.R'
        'EpivizChart-class.R' 'restartEpiviz.R' 'settings_utils.R'
        'startEpiviz.R' 'vignette_data.R'
biocViews: Visualization, Infrastructure, GUI
Video: https://www.youtube.com/watch?v=099c4wUxozA
RoxygenNote: 7.1.0
git_url: https://git.bioconductor.org/packages/epivizr
git_branch: RELEASE_3_23
git_last_commit: 9cd98d7
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:59:15 UTC; biocbuild
